#!/usr/bin/env python

import cdblib
import os
import re
import requests
import tomllib

BLOCKLISTS_CONF  = "/etc/dnsdist/blocklists.toml"
BLOCKLIST_DB     = "/var/lib/dnsdist/blocklist.cdb"
BLOCKLIST_DB_NEW = BLOCKLIST_DB + ".new"
ALLOWLIST_DB     = "/var/lib/dnsdist/allowlist.cdb"
ALLOWLIST_DB_NEW = ALLOWLIST_DB + ".new"

comment = re.compile(r"^((\s|\t)*#|(\s|\t)*$)")

def add_domains(db, domains, check_comments):
    if len(domains) > 0:
        i = 0
        for domain in domains:
            if check_comments and comment.match(domain):
                continue
            db.put(domain)
            i += 1
        if i > 0:
            print(f"Domains added: {i}")
        else:
            if check_comments:
                print("No domains fetched")
    else:
        print("The list of domains is empty")

def commit_list(tmp, tgt):
    os.chmod(tmp, 0o400)
    os.rename(tmp, tgt)

with open(BLOCKLISTS_CONF, "rb") as config_file:
    config = tomllib.load(config_file)

sources = config["sources"]

db = cdblib.Writer(open(BLOCKLIST_DB_NEW, "wb"))

if len(sources) > 0:
    for source in sources:
        print(f"Processing {source}...")
        content = requests.get(source, timeout=10)
        if content.status_code == 200:
            domains = content.content.decode("utf-8").splitlines()
            add_domains(db, domains, True)
        else:
            print("Source is unavailable")
else:
    print("No sources available")

print("Processing manual blocklist...")
add_domains(db, config["blocklist"], False)

db.finalize()

print("Processing manual allowlist...")

db = cdblib.Writer(open(ALLOWLIST_DB_NEW, "wb"))
add_domains(db, config["allowlist"], False)
db.finalize()

print("Installing new lists...")

commit_list(BLOCKLIST_DB_NEW, BLOCKLIST_DB)
commit_list(ALLOWLIST_DB_NEW, ALLOWLIST_DB)

print("Done.")
